#! /usr/bin/env python
'''
Compare the given config files, return information shared by all files 

$Id$
'''
from __future__ import print_function

import sys
import io
import argparse
import package_info
import re
from feedback import die

from configobj import Section
from expconfig import ConfigObj

# Extract overlap of two config objects to new config object
def findConfigShared(c1, c2, cnew, depth = 0):
    if depth == 0:
        main = cnew
    else:
        main = cnew.main
    for k in c1:
        if k in c2:
            if type(c1[k]) is Section:
                cnew[k] = Section(cnew, depth+1, main)
                findConfigShared(c1[k],c2[k], cnew[k], depth+1)
            else: 
                if c1[k] == c2[k]:
                    cnew[k] = c1[k]
    #??? TODO: save differences separately?


#
# Main routine
#

# Check command line

command_line = argparse.ArgumentParser(description=__doc__.split('\n', 1)[0])
# TODO: print differences option,  ...
command_line.add_argument('--indent-string', default='  ', help='set indent string [%(default)s]')
command_line.add_argument('--inline-comments' , '-c', action='store_true',
                          help='compact white space before inline comments'
                               ' (BETA)')
command_line.add_argument('--trailing-space' , '-t', action='store_true',
                          help='remove white space at end of lines')
command_line.add_argument('config1')
command_line.add_argument('config2')
command_line.add_argument('configs', nargs='*', metavar='config',
                          help='original configuration file names')

command_line.add_argument('-V', '--version', action='version',
                          version=package_info.version)
args = command_line.parse_args()


# File handling

try:
    config_files = [args.config1, args.config2] + args.configs
    config_data = list()
    for i, config_file in enumerate(config_files):
        config_data.append(ConfigObj(config_file, 
                interpolation=False, file_error=True))
except IOError as error:
    die(error.message)

# Compare configs

for i in range(len(config_files)-1):
    shared_config_data = ConfigObj(indent_type=args.indent_string)
    findConfigShared(config_data[i],config_data[i+1],shared_config_data)
    config_data[i+1] = shared_config_data

# Ready to roll out

lines = io.BytesIO()
shared_config_data.write(lines)

lines.seek(0)
for line in io.TextIOWrapper(lines):
    if args.inline_comments: line = re.sub(r' = (.*?)  #', r' = \1 #', line)
    if args.trailing_space:
        print(line.rstrip())
    else:
        print(line, end='')
